-------------------------------------------------------------------------------------------
-- SUMMONING SYSTEM
-- Version 1.0
-------------------------------------------------------------------------------------------
-- the summoning namespace
Summoning = {}

-------------------------------------------------------------------------------------------
-- CATEGORY TYPES (Key)(String)
-- Version 1.0
-------------------------------------------------------------------------------------------
-- used in initialize player summon table function
Summoning.CategoryTypes =
{
	"AirMagic",
	"DarkMagic",
	"EarthMagic",
	"FireMagic",
	"WaterMagic",
}

-------------------------------------------------------------------------------------------
-- CATEGORY TYPES DEFINITIONS (Key)(Table)
-- Version 1.0
-------------------------------------------------------------------------------------------
-- a table containing the skill category definitions
-- the Max table will contain your maximum for each summon type
-- the supporting functions use these tables for the calculations
-- upon player login, their tables get regenerated off this base table
-- this is because if a player logs out, all summons go poof with them.
Summoning.CategoryTypesDefinitions =
{
	AirMagic = { Max = { AirElemental = 1, AirMephit = 3, }, },
	DarkMagic = { Max = { BoneWarriors = 3, RagingSpirits = 8, }, },
	EarthMagic = { Max = { EarthElemental = 1, EarthMephit = 3, }, },
	FireMagic = { Max = { FireElemental = 1, FireMephit = 3, }, },
	WaterMagic = { Max = { WaterElemental = 1, WaterMephit = 3, }, },		
}

-------------------------------------------------------------------------------------------
-- SUPPORTING FUNCTIONS
-- Version 1.0
-------------------------------------------------------------------------------------------
--- returns true if you can summon this type of mobile
-- @param playerObject - the player to get the summoning info from
-- @param skillName -
-- @param mobileCategory -
function CanAddSummon(playerObject,skillName,mobileCategory)
	local summonTable = playerObject:GetObjVar("SummonTable") or nil
	if(summonTable) then
		-- can we summon another of this type?
		if(#summonTable[skillName][mobileCategory] < GetMaxSummons(skillName,mobileCategory)) then
			return true
		else
			-- if we are over our limit, then remove the first instance in the table
			local mobile = summonTable[skillName][mobileCategory][1] or nil
			if(mobile) then
				table.remove(summonTable[skillName][mobileCategory],1)
				PlayEffectAtLoc("CloakEffect",mobile:GetLoc())
				mobile:Destroy()
				playerObject:SetObjVar("SummonTable",summonTable)
				return true
			end
		end
	end
	return false
end

--- destroys all current summoned minions the player has
--- this function is called from the player.lua logout event
-- @param playerObject - the player object you wish to remove the summons for
function DestroyAllPlayerSummons(playerObject)
	local summonTable = playerObject:GetObjVar("SummonTable")
	if(summonTable) then
		for skillName,value1 in pairs(summonTable) do
			for mobType,value2 in pairs(summonTable[skillName]) do
				for index,value3 in pairs(summonTable[skillName][mobType]) do
					if(summonTable[skillName][mobType][index]) then
						summonTable[skillName][mobType][index]:Destroy()
					end
				end
			end
		end
	end
end

--- returns the max you can summon from a given category and summon type
-- @param skillName -
-- @param mobileCategory -
function GetMaxSummons(skillName,mobileCategory)
	return (Summoning.CategoryTypesDefinitions[skillName].Max[mobileCategory] or 0)
end

--- this will fire in the player.lua override in the loadfrombackup event
--- it sets up the table entries for summoning.
-- @param playerObject - the player to setup 
function InitializePlayerSummonTable(playerObject)
	local tempTable = {}
	for key,value in pairs(Summoning.CategoryTypes) do
		-- create the initial table
		tempTable[value] = {}
		-- add an entry into each table for the summon types
		for k,v in pairs(Summoning.CategoryTypesDefinitions[value].Max) do
			tempTable[value][k] = {}
		end
	end
	-- update the player summon table
	playerObject:SetObjVar("SummonTable",tempTable)
end

--- adds the summon into the correct table for the use of the summoning system
-- @param playerObject - the player to get the summoning info from
-- @param skillName - category type
-- @param mobileCategory - mob table type
-- @param summonedMobile - the summoned object reference
function PlayerAddSummon(playerObject,skillName,mobileCategory,summonedMobile)
	local summonTable = playerObject:GetObjVar("SummonTable")
	if(summonTable) then
		table.insert(summonTable[skillName][mobileCategory],summonedMobile)
		playerObject:SetObjVar("SummonTable",summonTable)
	end
end

--- removes the summon from the correct table for the use of the summoning system
-- @param playerObject - the player to get the summoning info from
-- @param skillName - category type
-- @param mobileCategory - mob table type
-- @param summonedMobile - the summoned object reference
function PlayerRemoveSummon(playerObject,skillName,mobileCategory,summonedMobile)
	local summonTable = playerObject:GetObjVar("SummonTable")
	if(summonTable) then
		for key,value in pairs(summonTable[skillName][mobileCategory]) do
			if(value == summonedMobile) then
				table.remove(summonTable[skillName][mobileCategory],key)
				playerObject:SetObjVar("SummonTable",summonTable)
				return
			end
		end
	end
end

